import pygame
from pygame.time import Clock
from pygame.mixer import Sound
from time import sleep, time
from base import *

clock = Clock() # Make the clock for fps regulation

pygame.init() # Instance pygame

# THE VARIABLES OMG
window = pygame.display.set_mode((800, 600)) # Set the window dimensions
background = pygame.image.load('scape.png') # Get background image
pygame.display.set_caption('Cops and Bunnies!') # Set image caption
font = pygame.font.Font('font.ttf', 32) # Add the font
camera = Camera() # Make a camera
bunny = Bunny(window, 0, 0, camera) # Make the bunny
maxy = 0 # Bunny's max y
generator = PlatformGenerator(window, 80, 200, range(20, 90), 50, camera) # Make the platform generator
running = True
bullets = [] # Make the bullet list
eagles = [] # Make the eagle list
score = 0 # Set the score
spawn = SpawnGround(window, 0, 100, camera) # Make the spawn ground
music = Sound('music.wav') # Make the music
bunnyKnownDeath = False
sinceMusicStarted = time()
sinceLastEagle = time()
sinceLastDeath = time()
bltgun = None
bltguntime = 0
mouse = Mouse(camera) # Make the mouse
music.set_volume(0.54) # Set the volume of the music to almost half
music.play() # Play the music
platforms = generator.generate() # Generate the platforms and make them into a list
last = platforms[-1].x, platforms[-1].y
platforms.append(spawn) # Add the spawn ground into the list

while running: # Game Loop
    window.blit(background, (0, 0)) # Reset the screen
    clock.tick(69) # Noice FPS
    for event in pygame.event.get(): # Get events
        bunny.events(event) # Process events for the bunny
        if event.type == pygame.QUIT:
            running = False # Quit the game loop
        if event.type == pygame.MOUSEBUTTONDOWN:
            bullet = bunny.shoot(mouse, eagles) # Try to shoot a bullet
            if bullet is not None: # If it returns a bullet
                bltgun = pygame.transform.scale(pygame.image.load('gun.png'), (80, 80))
                bltgun = pygame.transform.rotate(bltgun, -atan2(bullet.velocity[1], bullet.velocity[0]) * 180 / pi)
                bltguntime = 69 # Set the amount of time
                bullets.append(bullet) # Add it to the list of bullets
    if sinceLastEagle + 5 < time() and len(eagles) < 3:
        eagles.append(Eagle(window, bunny, bunny.x - 450, bunny.y + random.randint(-150, 0), camera)) # Make an eagle if the cooldown is gone and there is less than 3
        sinceLastEagle = time() # Reset the cooldown
    for i, eagle in enumerate(eagles):
        eagle.update() # Update the eagles
        if eagle.dead:
            del eagles[i] # Remove eagle from the list if they're dead
    bunny.checkPlatforms(platforms) # Check if the bunny is colliding with platforms
    if bunny.x + 700 > last[0]:
        generator.startX = last[0]
        generator.startY = last[1]
        platforms += generator.generate()
        last = platforms[-1].x, platforms[-1].y
    for i in platforms: 
        i.draw() # Draw every platform
    if bunny.y < maxy:
        maxy = bunny.y # Set the max y the bunny has been
    if bunny.y > maxy + 300:
        bunny.dead = True # Kill the bunny
    bunny.update() # Update the bunny
    mouse.update() # Update the mouse
    spawn.draw() # Draw the spawn ground
    for i in bullets:
        i.update() # Update every bullet
    if bunny.dead and not bunnyKnownDeath:
        sinceLastDeath = time() # Set last death
        bullets = [] # Remove all bullets
        bunnyKnownDeath = True # Let the game know the bunny is dead
    if sinceLastDeath + 2 < time() and bunny.dead:
        bunny.x = 0 # Reset bunny x
        bunny.y = 0 # Reset bunny y
        eagles = [] # Reset eagles
        platforms = [] # Remove all platforms
        platforms = generator.generate() # Reset the platforms
        last = platforms[-1].x, platforms[-1].y
        platforms.append(spawn) # Add spawn platforms to the list
        maxy = 0
        bunny.dead = False # Revive the bunny
        bunnyKnownDeath = False # Let the game know the bunny is alive
    elif bunny.dead:
        window.blit(font.render('You died! The Eagle Police station has caught up to you', 0, (0, 0, 0)), (0, 300)) # Render the text
    window.blit(font.render(f'Your score: {bunny.x / 10:.0f}', 0, (0, 0, 0)), (0, 0)) # Render the score
    if bltguntime > 0:
        if not bunny.dead:
            window.blit(bltgun, (bunny.x - camera.x + 14, bunny.y - camera.y)) # Render the gun
        bltguntime -= 1 # Reduce the time to render
    if sinceMusicStarted + music.get_length() < time(): # If the music is over
        music.play() # Play it again
        sinceMusicStarted = time()
    window.blit(font.render('A/D to move, space to jump', 0, (0, 0, 0)), (400, 0))
    pygame.display.update() # Update the display
pygame.quit() # Quit pygame